@@ -5,6 +5,12 @@ use crate::router::{routes::RoutePath, Router};
 use crate::scheduler::{Scheduler, TimerId, Topic};
 use crate::screen::touch::on_touch;
 use crate::watcher::configuration_file_updates;
+#[cfg(all(
+    feature = "audio",
+    not(target_os = "macos"),
+    not(target_os = "windows")
+))]
+use cpal::traits::{DeviceTrait, HostTrait, StreamTrait};
 use raw_window_handle::HasDisplayHandle;
 use rio_backend::clipboard::{Clipboard, ClipboardType};
 use rio_backend::config::colors::ColorRgb;
@@ -88,6 +94,76 @@ impl Application<'_> {
         )
     }
 
+    fn handle_visual_bell(&mut self, window_id: WindowId) {
+        if let Some(route) = self.router.routes.get_mut(&window_id) {
+            route.window.screen.renderer.trigger_visual_bell();
+
+            // Mark content as dirty to ensure render happens
+            route
+                .window
+                .screen
+                .ctx_mut()
+                .current_mut()
+                .renderable_content
+                .pending_update
+                .set_dirty();
+
+            // Force immediate render to show the bell
+            route.request_redraw();
+
+            // Schedule a render after the bell duration to clear it
+            let timer_id =
+                TimerId::new(Topic::Render, route.window.screen.ctx().current_route());
+            let event = EventPayload::new(RioEventType::Rio(RioEvent::Render), window_id);
+
+            // Schedule render to clear bell effect after visual bell duration
+            self.scheduler.schedule(
+                event,
+                crate::constants::BELL_DURATION,
+                false,
+                timer_id,
+            );
+        }
+    }
+
+    fn handle_audio_bell(&mut self) {
+        #[cfg(target_os = "macos")]
+        {
+            // Use system bell sound on macOS
+            unsafe {
+                #[link(name = "AppKit", kind = "framework")]
+                extern "C" {
+                    fn NSBeep();
+                }
+                NSBeep();
+            }
+        }
+
+        #[cfg(target_os = "windows")]
+        {
+            // Use MessageBeep on Windows with MB_OK (0x00000000) for default beep
+            unsafe {
+                windows_sys::Win32::System::Diagnostics::Debug::MessageBeep(0x00000000);
+            }
+        }
+
+        #[cfg(all(not(target_os = "macos"), not(target_os = "windows")))]
+        {
+            #[cfg(feature = "audio")]
+            {
+                std::thread::spawn(|| {
+                    if let Err(e) = play_bell_sound() {
+                        tracing::warn!("Failed to play bell sound: {}", e);
+                    }
+                });
+            }
+            #[cfg(not(feature = "audio"))]
+            {
+                tracing::debug!("Audio bell requested but audio feature is not enabled");
+            }
+        }
+    }
+
     pub fn run(
         &mut self,
         event_loop: EventLoop<EventPayload>,
@@ -428,6 +504,17 @@ impl ApplicationHandler<EventPayload> for Application<'_> {
                     }
                 }
             }
+            RioEventType::Rio(RioEvent::Bell) => {
+                // Handle visual bell
+                if self.config.bell.visual {
+                    self.handle_visual_bell(window_id);
+                }
+
+                // Handle audio bell
+                if self.config.bell.audio {
+                    self.handle_audio_bell();
+                }
+            }
             RioEventType::Rio(RioEvent::PrepareRender(millis)) => {
                 if let Some(route) = self.router.routes.get(&window_id) {
                     let timer_id = TimerId::new(
@@ -1363,3 +1450,75 @@ impl ApplicationHandler<EventPayload> for Application<'_> {
         std::process::exit(0);
     }
 }
+
+#[cfg(all(
+    feature = "audio",
+    not(target_os = "macos"),
+    not(target_os = "windows")
+))]
+fn play_bell_sound() -> Result<(), Box<dyn Error>> {
+    let host = cpal::default_host();
+    let device = host
+        .default_output_device()
+        .ok_or("No output device available")?;
+
+    let config = device.default_output_config()?;
+
+    match config.sample_format() {
+        cpal::SampleFormat::F32 => run_bell::<f32>(&device, &config.into()),
+        cpal::SampleFormat::I16 => run_bell::<i16>(&device, &config.into()),
+        cpal::SampleFormat::U16 => run_bell::<u16>(&device, &config.into()),
+        _ => Err("Unsupported sample format".into()),
+    }
+}
+
+#[cfg(all(
+    feature = "audio",
+    not(target_os = "macos"),
+    not(target_os = "windows")
+))]
+fn run_bell<T>(
+    device: &cpal::Device,
+    config: &cpal::StreamConfig,
+) -> Result<(), Box<dyn Error>>
+where
+    T: cpal::Sample + cpal::SizedSample + cpal::FromSample<f32>,
+{
+    let sample_rate = config.sample_rate.0 as f32;
+    let channels = config.channels as usize;
+    let duration_secs = crate::constants::BELL_DURATION.as_secs_f32();
+    let total_samples = (sample_rate * duration_secs) as usize;
+
+    let mut sample_clock = 0f32;
+    let mut samples_played = 0usize;
+
+    let stream = device.build_output_stream(
+        config,
+        move |data: &mut [T], _: &cpal::OutputCallbackInfo| {
+            for frame in data.chunks_mut(channels) {
+                if samples_played >= total_samples {
+                    for sample in frame.iter_mut() {
+                        *sample = T::from_sample(0.0);
+                    }
+                } else {
+                    let value = (sample_clock * 440.0 * 2.0 * std::f32::consts::PI
+                        / sample_rate)
+                        .sin()
+                        * 0.2;
+                    for sample in frame.iter_mut() {
+                        *sample = T::from_sample(value);
+                    }
+                    sample_clock += 1.0;
+                    samples_played += 1;
+                }
+            }
+        },
+        |err| tracing::error!("Audio stream error: {}", err),
+        None,
+    )?;
+
+    stream.play()?;
+    std::thread::sleep(crate::constants::BELL_DURATION);
+
+    Ok(())
+}
